@@ -10,6 +10,7 @@ from zope.index.text.textindex import TextIndex
 from zope.index.text.cosineindex import CosineIndex
 from zope.index.text.lexicon import Lexicon
 from zope.index.text.lexicon import Splitter
+from zope.index.text.stopdict import get_stopdict
 import time
 import dedupe.tfidf as tfidf
 
@@ -19,7 +20,10 @@ class Blocker:
     '''Takes in a record and returns all blocks that record belongs to'''
     def __init__(self, 
                  predicates, 
-                 stop_words = defaultdict(set)) :
+                 stop_words = None) :
+
+        if stop_words is None :
+            stop_words = defaultdict(lambda : set(get_stopdict()))
 
         self.predicates = predicates
 
@@ -29,7 +33,7 @@ class Blocker:
 
         for full_predicate in predicates :
             for predicate in full_predicate :
-                if predicate.type == "TfidfPredicate" :
+                if hasattr(predicate, 'canopy') :
                     self.tfidf_fields[predicate.field].add(predicate)
 
     #@profile
@@ -53,6 +57,16 @@ class Blocker:
                              {'iteration' :i,
                               'elapsed' :time.time() - start_time})
 
+        self._resetCanopies()
+
+
+    def _resetCanopies(self) :
+        # clear canopies to reduce memory usage
+        for predicate_set in self.tfidf_fields.values() :
+            for predicate in predicate_set :
+                predicate.canopy = {}
+
+
 
 class DedupeBlocker(Blocker) :
     
@@ -64,18 +78,24 @@ class DedupeBlocker(Blocker) :
 
         stop_word_remover = CustomStopWordRemover(self.stop_words[field])
 
+
         indices = {}
         for predicate in self.tfidf_fields[field] :
             indices[predicate] = TextIndex(Lexicon(splitter, stop_word_remover))
             indices[predicate].index = CosineIndex(indices[predicate].lexicon)
+            pipeline = indices[predicate].lexicon._pipeline
+            stringify = predicate.stringify
 
         index_to_id = {}
         base_tokens = {}
 
-
         for i, (record_id, doc) in enumerate(data, 1) :
+            doc = stringify(doc)
             index_to_id[i] = record_id
-            base_tokens[i] = splitter.process([doc])
+            last = [doc]
+            for each in pipeline :
+                last = each.process(last)
+            base_tokens[i] = ' OR '.join(last)
             for index in indices.values() :
                 index.index_doc(i, doc)
 
@@ -104,6 +124,8 @@ class RecordLinkBlocker(Blocker) :
         for predicate in self.tfidf_fields[field] :
             indices[predicate] = TextIndex(Lexicon(splitter, stop_word_remover))
             indices[predicate].index = CosineIndex(indices[predicate].lexicon)
+            pipeline = indices[predicate].lexicon._pipeline
+            stringify = predicate.stringify
 
         index_to_id = {}
         base_tokens = {}
@@ -111,11 +133,16 @@ class RecordLinkBlocker(Blocker) :
         i = 1
 
         for record_id, doc in data_1 :
+            doc = stringify(doc)
             index_to_id[i] = record_id
-            base_tokens[i] = splitter.process([doc])
+            last = [doc]
+            for each in pipeline :
+                last = each.process(last)
+            base_tokens[i] = ' OR '.join(last)
             i += 1
 
         for record_id, doc in data_2  :
+            doc = stringify(doc)
             index_to_id[i] = record_id
             for index in indices.values() :
                 index.index_doc(i, doc)
@@ -154,9 +181,6 @@ def blockTraining(training_pairs,
         coverage = DedupeCoverage(predicate_set,
                                   training_dupes | training_distinct)
 
-
-    print len(coverage.overlap.keys())
-
     # Compound Predicates
     compound_predicates = itertools.combinations(coverage.overlap, 2)
     intersection = set.intersection
@@ -319,6 +343,7 @@ class Coverage(object) :
                 if len(block) < 100 :
                     del self.blocks[predicate][field_pred]
 
+
     def predicateCoverage(self,
                           predicate_set,
                           pairs) :
@@ -335,7 +360,6 @@ class Coverage(object) :
 
 class DedupeCoverage(Coverage) :
     def __init__(self, predicate_set, pairs) :
-        self.stop_words = {}
 
         records = set(itertools.chain(*pairs))
         
@@ -350,16 +374,15 @@ class DedupeCoverage(Coverage) :
                     for record_id, record
                     in id_records.items()]
             stop_words = stopWords(data)
-            blocker.stop_words[field] = stop_words
-            self.stop_words[field] = stop_words
+            blocker.stop_words[field].update(stop_words)
             blocker.tfIdfBlock(data, field)
-        
+
+        self.stop_words = blocker.stop_words
         self.coveredBy(blocker.predicates, record_ids, pairs)
+        blocker._resetCanopies()
 
 class RecordLinkCoverage(Coverage) :
     def __init__(self, predicate_set, pairs) :
-        self.stop_words = {}
-
         
         data_1 = set([])
         data_2 = set([])
@@ -398,13 +421,14 @@ class RecordLinkCoverage(Coverage) :
                         in id_records_2.items()]
 
             stop_words = stopWords(fields_2)
-            blocker.stop_words[field] = stop_words
-            self.stop_words[field] = stop_words
-
+            blocker.stop_words[field].update(stop_words)
+ 
             blocker.tfIdfBlock(fields_1, fields_2, field)
 
+        self.stop_words = blocker.stop_words
         self.coveredBy(blocker.predicates, record_ids, pairs)
-
+        blocker._resetCanopies()
+        
 
 def stopWords(data) :
     index = TextIndex(Lexicon(Splitter()))
@@ -437,6 +461,12 @@ class Predicate(object) :
     def __repr__(self) :
         return "%s: %s" % (self.type, self.__name__)
 
+    def __hash__(self) :
+        return hash(repr(self))
+
+    def __eq__(self, other) :
+        return repr(self) == repr(other)
+
 class SimplePredicate(Predicate) :
     type = "SimplePredicate"
 
@@ -466,7 +496,9 @@ class TfidfPredicate(Predicate):
         else :
             return ()
 
-        return call
+
+    def stringify(self, doc) :
+        return doc
 
 
     def __getstate__(self):
@@ -474,6 +506,12 @@ class TfidfPredicate(Predicate):
         result['canopy'] = {}
         return result
 
+class TfidfSetPredicate(TfidfPredicate) :
+    type = "TfidfPredicate"
+
+    def stringify(self, doc) :
+        return ' '.join('_'.join(str(each).split()) for each in doc)
+
 
 class CompoundPredicate(Predicate) :
     type = "CompoundPredicate"
@@ -498,7 +536,7 @@ class CompoundPredicate(Predicate) :
 
 class CustomStopWordRemover(object):
     def __init__(self, stop_words) :
-        self.stop_words = stop_words.copy()
+        self.stop_words = stop_words
 
     def process(self, lst):
         return [w for w in lst if not w in self.stop_words]
